@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""HTTP error classes.
+"""HTTP error classes and other Falcon-specific errors.
 
 This module implements a collection of `falcon.HTTPError`
 specializations that can be raised to generate a 4xx or 5xx HTTP
@@ -43,6 +43,10 @@ from falcon.http_error import HTTPError, NoRepresentation, \
 import falcon.status_codes as status
 
 
+class HeaderNotSupported(ValueError):
+    """The specified header is not supported by this method."""
+
+
 class HTTPBadRequest(HTTPError):
     """400 Bad Request.
 
@@ -83,9 +87,9 @@ class HTTPBadRequest(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPBadRequest, self).__init__(status.HTTP_400, title,
-                                             description, **kwargs)
+                                             description, headers, **kwargs)
 
 
 class HTTPUnauthorized(HTTPError):
@@ -146,14 +150,15 @@ class HTTPUnauthorized(HTTPError):
 
     """
 
-    def __init__(self, title=None, description=None, challenges=None, **kwargs):
-        headers = kwargs.setdefault('headers', {})
+    def __init__(self, title=None, description=None, challenges=None, headers=None, **kwargs):
+        if headers is None:
+            headers = {}
 
         if challenges:
             headers['WWW-Authenticate'] = ', '.join(challenges)
 
         super(HTTPUnauthorized, self).__init__(status.HTTP_401, title,
-                                               description, **kwargs)
+                                               description, headers, **kwargs)
 
 
 class HTTPForbidden(HTTPError):
@@ -207,9 +212,9 @@ class HTTPForbidden(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPForbidden, self).__init__(status.HTTP_403, title,
-                                            description, **kwargs)
+                                            description, headers, **kwargs)
 
 
 class HTTPNotFound(OptionalRepresentation, HTTPError):
@@ -261,8 +266,8 @@ class HTTPNotFound(OptionalRepresentation, HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, **kwargs):
-        super(HTTPNotFound, self).__init__(status.HTTP_404, **kwargs)
+    def __init__(self, headers=None, **kwargs):
+        super(HTTPNotFound, self).__init__(status.HTTP_404, headers=headers, **kwargs)
 
 
 class HTTPMethodNotAllowed(OptionalRepresentation, HTTPError):
@@ -316,9 +321,10 @@ class HTTPMethodNotAllowed(OptionalRepresentation, HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, allowed_methods, **kwargs):
+    def __init__(self, allowed_methods, headers=None, **kwargs):
         new_headers = {'Allow': ', '.join(allowed_methods)}
         super(HTTPMethodNotAllowed, self).__init__(status.HTTP_405,
+                                                   headers=headers,
                                                    **kwargs)
         if not self.headers:
             self.headers = {}
@@ -373,10 +379,11 @@ class HTTPNotAcceptable(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, description=None, **kwargs):
+    def __init__(self, description=None, headers=None, **kwargs):
         super(HTTPNotAcceptable, self).__init__(status.HTTP_406,
                                                 'Media type not acceptable',
-                                                description, **kwargs)
+                                                description, headers,
+                                                **kwargs)
 
 
 class HTTPConflict(HTTPError):
@@ -430,9 +437,10 @@ class HTTPConflict(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPConflict, self).__init__(status.HTTP_409, title,
-                                           description, **kwargs)
+                                           description, headers,
+                                           **kwargs)
 
 
 class HTTPGone(OptionalRepresentation, HTTPError):
@@ -492,8 +500,8 @@ class HTTPGone(OptionalRepresentation, HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, **kwargs):
-        super(HTTPGone, self).__init__(status.HTTP_410, **kwargs)
+    def __init__(self, headers=None, **kwargs):
+        super(HTTPGone, self).__init__(status.HTTP_410, headers=headers, **kwargs)
 
 
 class HTTPLengthRequired(HTTPError):
@@ -537,9 +545,10 @@ class HTTPLengthRequired(HTTPError):
             support request or to help them when searching for knowledge
             base articles related to this error (default ``None``).
     """
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPLengthRequired, self).__init__(status.HTTP_411,
-                                                 title, description, **kwargs)
+                                                 title, description,
+                                                 headers, **kwargs)
 
 
 class HTTPPreconditionFailed(HTTPError):
@@ -585,13 +594,14 @@ class HTTPPreconditionFailed(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPPreconditionFailed, self).__init__(status.HTTP_412, title,
-                                                     description, **kwargs)
+                                                     description, headers,
+                                                     **kwargs)
 
 
-class HTTPRequestEntityTooLarge(HTTPError):
-    """413 Request Entity Too Large.
+class HTTPPayloadTooLarge(HTTPError):
+    """413 Payload Too Large.
 
     The server is refusing to process a request because the request
     payload is larger than the server is willing or able to process.
@@ -606,7 +616,7 @@ class HTTPRequestEntityTooLarge(HTTPError):
     (See also: RFC 7231, Section 6.5.11)
 
     Keyword Args:
-        title (str): Error title (default '413 Request Entity Too Large').
+        title (str): Error title (default '413 Payload Too Large').
 
         description (str): Human-friendly description of the error, along with
             a helpful suggestion or two.
@@ -641,18 +651,20 @@ class HTTPRequestEntityTooLarge(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, retry_after=None, **kwargs):
-        headers = kwargs.setdefault('headers', {})
+    def __init__(self, title=None, description=None, retry_after=None, headers=None, **kwargs):
+        if headers is None:
+            headers = {}
 
         if isinstance(retry_after, datetime):
             headers['Retry-After'] = util.dt_to_http(retry_after)
         elif retry_after is not None:
             headers['Retry-After'] = str(retry_after)
 
-        super(HTTPRequestEntityTooLarge, self).__init__(status.HTTP_413,
-                                                        title,
-                                                        description,
-                                                        **kwargs)
+        super(HTTPPayloadTooLarge, self).__init__(status.HTTP_413,
+                                                  title,
+                                                  description,
+                                                  headers,
+                                                  **kwargs)
 
 
 class HTTPUriTooLong(HTTPError):
@@ -703,8 +715,10 @@ class HTTPUriTooLong(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
-        super(HTTPUriTooLong, self).__init__(status.HTTP_414, title, description, **kwargs)
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
+        super(HTTPUriTooLong, self).__init__(status.HTTP_414, title,
+                                             description, headers,
+                                             **kwargs)
 
 
 class HTTPUnsupportedMediaType(HTTPError):
@@ -749,9 +763,10 @@ class HTTPUnsupportedMediaType(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, description=None, **kwargs):
+    def __init__(self, description=None, headers=None, **kwargs):
         super(HTTPUnsupportedMediaType, self).__init__(
-            status.HTTP_415, 'Unsupported media type', description, **kwargs)
+            status.HTTP_415, 'Unsupported media type',
+            description, headers, **kwargs)
 
 
 class HTTPRangeNotSatisfiable(NoRepresentation, HTTPError):
@@ -776,8 +791,11 @@ class HTTPRangeNotSatisfiable(NoRepresentation, HTTPError):
             request. Used to set the Content-Range header.
     """
 
-    def __init__(self, resource_length):
-        headers = {'Content-Range': 'bytes */' + str(resource_length)}
+    def __init__(self, resource_length, headers=None):
+        if headers is None:
+            headers = {}
+        headers.setdefault('Content-Range', 'bytes */' + str(resource_length))
+
         super(HTTPRangeNotSatisfiable, self).__init__(status.HTTP_416,
                                                       headers=headers)
 
@@ -827,9 +845,9 @@ class HTTPUnprocessableEntity(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPUnprocessableEntity, self).__init__(status.HTTP_422, title,
-                                                      description, **kwargs)
+                                                      description, headers, **kwargs)
 
 
 class HTTPLocked(OptionalRepresentation, HTTPError):
@@ -872,9 +890,10 @@ class HTTPLocked(OptionalRepresentation, HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPLocked, self).__init__(status.HTTP_423, title,
-                                         description, **kwargs)
+                                         description, headers,
+                                         **kwargs)
 
 
 class HTTPFailedDependency(OptionalRepresentation, HTTPError):
@@ -916,9 +935,10 @@ class HTTPFailedDependency(OptionalRepresentation, HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPFailedDependency, self).__init__(status.HTTP_424, title,
-                                                   description, **kwargs)
+                                                   description, headers,
+                                                   **kwargs)
 
 
 class HTTPPreconditionRequired(HTTPError):
@@ -967,9 +987,10 @@ class HTTPPreconditionRequired(HTTPError):
             support request or to help them when searching for knowledge
             base articles related to this error (default ``None``).
     """
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPPreconditionRequired, self).__init__(status.HTTP_428, title,
-                                                       description, **kwargs)
+                                                       description, headers,
+                                                       **kwargs)
 
 
 class HTTPTooManyRequests(HTTPError):
@@ -1020,8 +1041,9 @@ class HTTPTooManyRequests(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, retry_after=None, **kwargs):
-        headers = kwargs.setdefault('headers', {})
+    def __init__(self, title=None, description=None, retry_after=None, headers=None, **kwargs):
+        if headers is None:
+            headers = {}
 
         if isinstance(retry_after, datetime):
             headers['Retry-After'] = util.dt_to_http(retry_after)
@@ -1031,6 +1053,7 @@ class HTTPTooManyRequests(HTTPError):
         super(HTTPTooManyRequests, self).__init__(status.HTTP_429,
                                                   title,
                                                   description,
+                                                  headers,
                                                   **kwargs)
 
 
@@ -1080,10 +1103,11 @@ class HTTPRequestHeaderFieldsTooLarge(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPRequestHeaderFieldsTooLarge, self).__init__(status.HTTP_431,
                                                               title,
                                                               description,
+                                                              headers,
                                                               **kwargs)
 
 
@@ -1140,9 +1164,11 @@ class HTTPUnavailableForLegalReasons(OptionalRepresentation, HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, **kwargs):
+    def __init__(self, title=None, headers=None, **kwargs):
         super(HTTPUnavailableForLegalReasons, self).__init__(status.HTTP_451,
-                                                             title, **kwargs)
+                                                             title,
+                                                             headers=headers,
+                                                             **kwargs)
 
 
 class HTTPInternalServerError(HTTPError):
@@ -1184,9 +1210,10 @@ class HTTPInternalServerError(HTTPError):
 
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPInternalServerError, self).__init__(status.HTTP_500, title,
-                                                      description, **kwargs)
+                                                      description, headers,
+                                                      **kwargs)
 
 
 class HTTPNotImplemented(HTTPError):
@@ -1235,9 +1262,10 @@ class HTTPNotImplemented(HTTPError):
 
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPNotImplemented, self).__init__(status.HTTP_501, title,
-                                                 description, **kwargs)
+                                                 description, headers,
+                                                 **kwargs)
 
 
 class HTTPBadGateway(HTTPError):
@@ -1279,9 +1307,10 @@ class HTTPBadGateway(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPBadGateway, self).__init__(status.HTTP_502, title,
-                                             description, **kwargs)
+                                             description, headers,
+                                             **kwargs)
 
 
 class HTTPServiceUnavailable(HTTPError):
@@ -1335,8 +1364,9 @@ class HTTPServiceUnavailable(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, retry_after=None, **kwargs):
-        headers = kwargs.setdefault('headers', {})
+    def __init__(self, title=None, description=None, retry_after=None, headers=None, **kwargs):
+        if headers is None:
+            headers = {}
 
         if isinstance(retry_after, datetime):
             headers['Retry-After'] = util.dt_to_http(retry_after)
@@ -1346,6 +1376,7 @@ class HTTPServiceUnavailable(HTTPError):
         super(HTTPServiceUnavailable, self).__init__(status.HTTP_503,
                                                      title,
                                                      description,
+                                                     headers,
                                                      **kwargs)
 
 
@@ -1390,9 +1421,10 @@ class HTTPGatewayTimeout(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPGatewayTimeout, self).__init__(status.HTTP_504, title,
-                                                 description, **kwargs)
+                                                 description, headers,
+                                                 **kwargs)
 
 
 class HTTPVersionNotSupported(HTTPError):
@@ -1441,9 +1473,10 @@ class HTTPVersionNotSupported(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPVersionNotSupported, self).__init__(status.HTTP_505, title,
-                                                      description, **kwargs)
+                                                      description, headers,
+                                                      **kwargs)
 
 
 class HTTPInsufficientStorage(HTTPError):
@@ -1489,9 +1522,10 @@ class HTTPInsufficientStorage(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPInsufficientStorage, self).__init__(status.HTTP_507, title,
-                                                      description, **kwargs)
+                                                      description, headers,
+                                                      **kwargs)
 
 
 class HTTPLoopDetected(HTTPError):
@@ -1534,9 +1568,10 @@ class HTTPLoopDetected(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPLoopDetected, self).__init__(status.HTTP_508, title,
-                                               description, **kwargs)
+                                               description, headers,
+                                               **kwargs)
 
 
 class HTTPNetworkAuthenticationRequired(HTTPError):
@@ -1591,10 +1626,11 @@ class HTTPNetworkAuthenticationRequired(HTTPError):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, title=None, description=None, **kwargs):
+    def __init__(self, title=None, description=None, headers=None, **kwargs):
         super(HTTPNetworkAuthenticationRequired, self).__init__(status.HTTP_511,
                                                                 title,
                                                                 description,
+                                                                headers,
                                                                 **kwargs)
 
 
@@ -1634,13 +1670,14 @@ class HTTPInvalidHeader(HTTPBadRequest):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, msg, header_name, **kwargs):
+    def __init__(self, msg, header_name, headers=None, **kwargs):
         description = ('The value provided for the {0} header is '
                        'invalid. {1}')
         description = description.format(header_name, msg)
 
         super(HTTPInvalidHeader, self).__init__('Invalid header value',
-                                                description, **kwargs)
+                                                description, headers,
+                                                **kwargs)
 
 
 class HTTPMissingHeader(HTTPBadRequest):
@@ -1678,12 +1715,13 @@ class HTTPMissingHeader(HTTPBadRequest):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, header_name, **kwargs):
+    def __init__(self, header_name, headers=None, **kwargs):
         description = 'The {0} header is required.'
         description = description.format(header_name)
 
         super(HTTPMissingHeader, self).__init__('Missing header value',
-                                                description, **kwargs)
+                                                description, headers,
+                                                **kwargs)
 
 
 class HTTPInvalidParam(HTTPBadRequest):
@@ -1724,12 +1762,13 @@ class HTTPInvalidParam(HTTPBadRequest):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, msg, param_name, **kwargs):
+    def __init__(self, msg, param_name, headers=None, **kwargs):
         description = 'The "{0}" parameter is invalid. {1}'
         description = description.format(param_name, msg)
 
         super(HTTPInvalidParam, self).__init__('Invalid parameter',
-                                               description, **kwargs)
+                                               description, headers,
+                                               **kwargs)
 
 
 class HTTPMissingParam(HTTPBadRequest):
@@ -1769,9 +1808,10 @@ class HTTPMissingParam(HTTPBadRequest):
             base articles related to this error (default ``None``).
     """
 
-    def __init__(self, param_name, **kwargs):
+    def __init__(self, param_name, headers=None, **kwargs):
         description = 'The "{0}" parameter is required.'
         description = description.format(param_name)
 
         super(HTTPMissingParam, self).__init__('Missing parameter',
-                                               description, **kwargs)
+                                               description, headers,
+                                               **kwargs)
