@@ -13,15 +13,13 @@ class MessagePackHandler(BaseHandler):
     (``bytes`` on Python 2/3, or ``str`` on Python 2).
 
     Note:
-        This handler requires the extra ``msgpack`` package, which must be
-        installed in addition to ``falcon`` from PyPI:
+        This handler requires the extra ``msgpack`` package (version 0.5.2
+        or higher), which must be installed in addition to ``falcon`` from
+        PyPI:
 
         .. code::
 
             $ pip install msgpack
-
-        Python 2.6 users will need to use the deprecated ``msgpack-python``
-        package instead, pinned to version 0.4.8.
     """
 
     def __init__(self):
@@ -29,21 +27,20 @@ class MessagePackHandler(BaseHandler):
 
         self.msgpack = msgpack
         self.packer = msgpack.Packer(
-            encoding='utf-8',
             autoreset=True,
             use_bin_type=True,
         )
 
-    def deserialize(self, raw):
+    def deserialize(self, stream, content_type, content_length):
         try:
             # NOTE(jmvrbanac): Using unpackb since we would need to manage
             # a buffer for Unpacker() which wouldn't gain us much.
-            return self.msgpack.unpackb(raw, encoding='utf-8')
+            return self.msgpack.unpackb(stream.read(), raw=False)
         except ValueError as err:
             raise errors.HTTPBadRequest(
                 'Invalid MessagePack',
                 'Could not parse MessagePack body - {0}'.format(err)
             )
 
-    def serialize(self, media):
+    def serialize(self, media, content_type):
         return self.packer.pack(media)
