@@ -20,14 +20,9 @@ Conversely, the `uri` module must be imported explicitly::
 
 """
 
-try:
-    import ujson as json  # NOQA
-except ImportError:
-    import json  # NOQA
+import json  # NOQA
 
 # Hoist misc. utils
-from falcon.util import structures
+from falcon.util.structures import *  # NOQA
 from falcon.util.misc import *  # NOQA
 from falcon.util.time import *  # NOQA
-
-CaseInsensitiveDict = structures.CaseInsensitiveDict
