@@ -11,16 +11,19 @@ def application(environ, start_response):
 
     body += '}\n\n'
 
-    return [body.encode('utf-8')]
+    if not isinstance(body, bytes):
+        body = body.encode('utf-8')
+
+    return [body]
+
 
 app = application
 
 
 if __name__ == '__main__':
-    # import eventlet.wsgi
-    # import eventlet
-    # eventlet.wsgi.server(eventlet.listen(('localhost', 8000)), application)
-
     from wsgiref.simple_server import make_server
     server = make_server('localhost', 8000, application)
+
+    print('Listening on localhost:8000...')
+
     server.serve_forever()
