@@ -1,12 +1,8 @@
 from datetime import datetime
+import json
 
 import pytest
 
-try:
-    import ujson as json
-except ImportError:
-    import json
-
 import falcon
 import falcon.testing as testing
 
@@ -25,6 +21,12 @@ class CaptureResponseMiddleware(object):
         self.req_succeeded = req_succeeded
 
 
+class CaptureRequestMiddleware(object):
+
+    def process_request(self, req, resp):
+        self.req = req
+
+
 class RequestTimeMiddleware(object):
 
     def process_request(self, req, resp):
@@ -47,7 +49,11 @@ class TransactionIdMiddleware(object):
         global context
         context['transaction_id'] = 'unique-req-id'
 
-    def process_response(self, req, resp, resource):
+    def process_resource(self, req, resp, resource, params):
+        global context
+        context['resource_transaction_id'] = 'unique-req-id-2'
+
+    def process_response(self, req, resp, resource, req_succeeded):
         pass
 
 
@@ -56,20 +62,20 @@ class ExecutedFirstMiddleware(object):
     def process_request(self, req, resp):
         global context
         context['executed_methods'].append(
-            '{0}.{1}'.format(self.__class__.__name__, 'process_request'))
+            '{}.{}'.format(self.__class__.__name__, 'process_request'))
 
     def process_resource(self, req, resp, resource, params):
         global context
         context['executed_methods'].append(
-            '{0}.{1}'.format(self.__class__.__name__, 'process_resource'))
+            '{}.{}'.format(self.__class__.__name__, 'process_resource'))
 
     # NOTE(kgriffs): This also tests that the framework can continue to
     # call process_response() methods that do not have a 'req_succeeded'
     # arg.
-    def process_response(self, req, resp, resource):
+    def process_response(self, req, resp, resource, req_succeeded):
         global context
         context['executed_methods'].append(
-            '{0}.{1}'.format(self.__class__.__name__, 'process_response'))
+            '{}.{}'.format(self.__class__.__name__, 'process_response'))
 
         context['req'] = req
         context['resp'] = resp
@@ -86,6 +92,24 @@ class RemoveBasePathMiddleware(object):
         req.path = req.path.replace('/base_path', '', 1)
 
 
+class ResponseCacheMiddlware(object):
+
+    PROCESS_REQUEST_CACHED_BODY = {'cached': True}
+    PROCESS_RESOURCE_CACHED_BODY = {'cached': True, 'resource': True}
+
+    def process_request(self, req, resp):
+        if req.path == '/cached':
+            resp.media = self.PROCESS_REQUEST_CACHED_BODY
+            resp.complete = True
+            return
+
+    def process_resource(self, req, resp, resource, params):
+        if req.path == '/cached/resource':
+            resp.media = self.PROCESS_RESOURCE_CACHED_BODY
+            resp.complete = True
+            return
+
+
 class AccessParamsMiddleware(object):
 
     def process_resource(self, req, resp, resource, params):
@@ -104,6 +128,15 @@ class MiddlewareClassResource(object):
         raise falcon.HTTPForbidden(falcon.HTTP_403, 'Setec Astronomy')
 
 
+class EmptySignatureMiddleware(object):
+
+    def process_request(self):
+        pass
+
+    def process_response(self):
+        pass
+
+
 class TestMiddleware(object):
     def setup_method(self, method):
         # Clear context
@@ -158,10 +191,12 @@ class TestRequestTimeMiddleware(TestMiddleware):
         with pytest.raises(Exception):
             client.simulate_request(path=TEST_ROUTE)
 
-    def test_log_get_request(self):
+    @pytest.mark.parametrize('independent_middleware', [True, False])
+    def test_log_get_request(self, independent_middleware):
         """Test that Log middleware is executed"""
         global context
-        app = falcon.API(middleware=[RequestTimeMiddleware()])
+        app = falcon.API(middleware=[RequestTimeMiddleware()],
+                         independent_middleware=independent_middleware)
 
         app.add_route(TEST_ROUTE, MiddlewareClassResource())
         client = testing.TestClient(app)
@@ -198,10 +233,20 @@ class TestTransactionIdMiddleware(TestMiddleware):
 
 
 class TestSeveralMiddlewares(TestMiddleware):
-    def test_generate_trans_id_and_time_with_request(self):
+    @pytest.mark.parametrize('independent_middleware', [True, False])
+    def test_generate_trans_id_and_time_with_request(self, independent_middleware):
+        # NOTE(kgriffs): We test both so that we can cover the code paths
+        # where only a single middleware method is implemented by a
+        # component.
+        creq = CaptureRequestMiddleware()
+        cresp = CaptureResponseMiddleware()
+
         global context
-        app = falcon.API(middleware=[TransactionIdMiddleware(),
-                                     RequestTimeMiddleware()])
+        app = falcon.API(independent_middleware=independent_middleware,
+                         middleware=[TransactionIdMiddleware(),
+                                     RequestTimeMiddleware(),
+                                     creq,
+                                     cresp])
 
         app.add_route(TEST_ROUTE, MiddlewareClassResource())
         client = testing.TestClient(app)
@@ -232,7 +277,8 @@ class TestSeveralMiddlewares(TestMiddleware):
 
     def test_middleware_execution_order(self):
         global context
-        app = falcon.API(middleware=[ExecutedFirstMiddleware(),
+        app = falcon.API(independent_middleware=False,
+                         middleware=[ExecutedFirstMiddleware(),
                                      ExecutedLastMiddleware()])
 
         app.add_route(TEST_ROUTE, MiddlewareClassResource())
@@ -284,11 +330,11 @@ class TestSeveralMiddlewares(TestMiddleware):
         context['req_succeeded'] = []
 
         class RaiseStatusMiddleware(object):
-            def process_response(self, req, resp, resource):
+            def process_response(self, req, resp, resource, req_succeeded):
                 raise falcon.HTTPStatus(falcon.HTTP_201)
 
         class RaiseErrorMiddleware(object):
-            def process_response(self, req, resp, resource):
+            def process_response(self, req, resp, resource, req_succeeded):
                 raise falcon.HTTPError(falcon.HTTP_748)
 
         class ProcessResponseMiddleware(object):
@@ -351,7 +397,7 @@ class TestSeveralMiddlewares(TestMiddleware):
                                      RequestTimeMiddleware(),
                                      RaiseErrorMiddleware()])
 
-        def handler(ex, req, resp, params):
+        def handler(req, resp, ex, params):
             context['error_handler'] = True
 
         app.add_error_handler(Exception, handler)
@@ -381,7 +427,7 @@ class TestSeveralMiddlewares(TestMiddleware):
                                      RaiseErrorMiddleware(),
                                      RequestTimeMiddleware()])
 
-        def handler(ex, req, resp, params):
+        def handler(req, resp, ex, params):
             context['error_handler'] = True
 
         app.add_error_handler(Exception, handler)
@@ -395,7 +441,7 @@ class TestSeveralMiddlewares(TestMiddleware):
         assert 'transaction_id' in context
         assert 'start_time' not in context
         assert 'mid_time' not in context
-        assert 'end_time' not in context
+        assert 'end_time' in context
         assert 'error_handler' in context
 
     def test_order_mw_executed_when_exception_in_resp(self):
@@ -411,7 +457,7 @@ class TestSeveralMiddlewares(TestMiddleware):
                                      RaiseErrorMiddleware(),
                                      ExecutedLastMiddleware()])
 
-        def handler(ex, req, resp, params):
+        def handler(req, resp, ex, params):
             pass
 
         app.add_error_handler(Exception, handler)
@@ -446,7 +492,7 @@ class TestSeveralMiddlewares(TestMiddleware):
                                      RaiseErrorMiddleware(),
                                      ExecutedLastMiddleware()])
 
-        def handler(ex, req, resp, params):
+        def handler(req, resp, ex, params):
             pass
 
         app.add_error_handler(Exception, handler)
@@ -480,7 +526,7 @@ class TestSeveralMiddlewares(TestMiddleware):
                                      RaiseErrorMiddleware(),
                                      ExecutedLastMiddleware()])
 
-        def handler(ex, req, resp, params):
+        def handler(req, resp, ex, params):
             pass
 
         app.add_error_handler(Exception, handler)
@@ -493,6 +539,7 @@ class TestSeveralMiddlewares(TestMiddleware):
         # Any mw is executed now...
         expectedExecutedMethods = [
             'ExecutedFirstMiddleware.process_request',
+            'ExecutedLastMiddleware.process_response',
             'ExecutedFirstMiddleware.process_response'
         ]
         assert expectedExecutedMethods == context['executed_methods']
@@ -511,7 +558,7 @@ class TestSeveralMiddlewares(TestMiddleware):
                                      RaiseErrorMiddleware(),
                                      ExecutedLastMiddleware()])
 
-        def handler(ex, req, resp, params):
+        def handler(req, resp, ex, params):
             pass
 
         app.add_error_handler(Exception, handler)
@@ -542,7 +589,7 @@ class TestSeveralMiddlewares(TestMiddleware):
                                      RaiseErrorMiddleware(),
                                      ExecutedLastMiddleware()])
 
-        def handler(ex, req, resp, params):
+        def handler(req, resp, ex, params):
             pass
 
         app.add_error_handler(Exception, handler)
@@ -576,7 +623,7 @@ class TestSeveralMiddlewares(TestMiddleware):
                                      RaiseErrorMiddleware(),
                                      ExecutedLastMiddleware()])
 
-        def handler(ex, req, resp, params):
+        def handler(req, resp, ex, params):
             pass
 
         app.add_error_handler(Exception, handler)
@@ -614,7 +661,9 @@ class TestRemoveBasePathMiddleware(TestMiddleware):
 
 
 class TestResourceMiddleware(TestMiddleware):
-    def test_can_access_resource_params(self):
+
+    @pytest.mark.parametrize('independent_middleware', [True, False])
+    def test_can_access_resource_params(self, independent_middleware):
         """Test that params can be accessed from within process_resource"""
         global context
 
@@ -622,7 +671,8 @@ class TestResourceMiddleware(TestMiddleware):
             def on_get(self, req, resp, **params):
                 resp.body = json.dumps(params)
 
-        app = falcon.API(middleware=AccessParamsMiddleware())
+        app = falcon.API(middleware=AccessParamsMiddleware(),
+                         independent_middleware=independent_middleware)
         app.add_route('/path/{id}', Resource())
         client = testing.TestClient(app)
         response = client.simulate_request(path='/path/22')
@@ -633,6 +683,18 @@ class TestResourceMiddleware(TestMiddleware):
         assert response.json == {'added': True, 'id': '22'}
 
 
+class TestEmptySignatureMiddleware(TestMiddleware):
+    def test_dont_need_params_in_signature(self):
+        """
+        Verify that we don't need parameters in the process_* signatures (for
+        side-effect-only middlewares, mostly). Makes no difference on py27
+        but does affect py36.
+
+        https://github.com/falconry/falcon/issues/1254
+        """
+        falcon.API(middleware=EmptySignatureMiddleware())
+
+
 class TestErrorHandling(TestMiddleware):
     def test_error_composed_before_resp_middleware_called(self):
         mw = CaptureResponseMiddleware()
@@ -670,3 +732,75 @@ class TestErrorHandling(TestMiddleware):
         response = client.simulate_request(path='/', method='POST')
         assert response.status == falcon.HTTP_201
         assert mw.resp.status == response.status
+
+
+class TestShortCircuiting(TestMiddleware):
+    def setup_method(self, method):
+        super(TestShortCircuiting, self).setup_method(method)
+
+    def _make_client(self, independent_middleware=True):
+        mw = [
+            RequestTimeMiddleware(),
+            ResponseCacheMiddlware(),
+            TransactionIdMiddleware(),
+        ]
+        app = falcon.API(middleware=mw, independent_middleware=independent_middleware)
+        app.add_route('/', MiddlewareClassResource())
+        app.add_route('/cached', MiddlewareClassResource())
+        app.add_route('/cached/resource', MiddlewareClassResource())
+
+        return testing.TestClient(app)
+
+    def test_process_request_not_cached(self):
+        response = self._make_client().simulate_get('/')
+        assert response.status == falcon.HTTP_200
+        assert response.json == _EXPECTED_BODY
+        assert 'transaction_id' in context
+        assert 'resource_transaction_id' in context
+        assert 'mid_time' in context
+        assert 'end_time' in context
+
+    @pytest.mark.parametrize('independent_middleware', [True, False])
+    def test_process_request_cached(self, independent_middleware):
+        response = self._make_client(independent_middleware).simulate_get('/cached')
+        assert response.status == falcon.HTTP_200
+        assert response.json == ResponseCacheMiddlware.PROCESS_REQUEST_CACHED_BODY
+
+        # NOTE(kgriffs): Since TransactionIdMiddleware was ordered after
+        # ResponseCacheMiddlware, the response short-circuiting should have
+        # skipped it.
+        assert 'transaction_id' not in context
+        assert 'resource_transaction_id' not in context
+
+        # NOTE(kgriffs): RequestTimeMiddleware only adds this in
+        # process_resource(), which should be skipped when
+        # ResponseCacheMiddlware sets resp.completed = True in
+        # process_request().
+        assert 'mid_time' not in context
+
+        # NOTE(kgriffs): Short-circuiting does not affect process_response()
+        assert 'end_time' in context
+
+    @pytest.mark.parametrize('independent_middleware', [True, False])
+    def test_process_resource_cached(self, independent_middleware):
+        response = self._make_client(independent_middleware).simulate_get('/cached/resource')
+        assert response.status == falcon.HTTP_200
+        assert response.json == ResponseCacheMiddlware.PROCESS_RESOURCE_CACHED_BODY
+
+        # NOTE(kgriffs): This should be present because it is added in
+        # process_request(), but the short-circuit does not occur until
+        # process_resource().
+        assert 'transaction_id' in context
+
+        # NOTE(kgriffs): Since TransactionIdMiddleware was ordered after
+        # ResponseCacheMiddlware, the response short-circuiting should have
+        # skipped it.
+        assert 'resource_transaction_id' not in context
+
+        # NOTE(kgriffs): RequestTimeMiddleware only adds this in
+        # process_resource(), which will not be skipped in this case because
+        # RequestTimeMiddleware is ordered before ResponseCacheMiddlware.
+        assert 'mid_time' in context
+
+        # NOTE(kgriffs): Short-circuiting does not affect process_response()
+        assert 'end_time' in context
