@@ -8,10 +8,10 @@ class TestRequestOptions(object):
     def test_option_defaults(self):
         options = RequestOptions()
 
-        assert not options.keep_blank_qs_values
+        assert options.keep_blank_qs_values
         assert not options.auto_parse_form_urlencoded
-        assert options.auto_parse_qs_csv
-        assert options.strip_url_path_trailing_slash
+        assert not options.auto_parse_qs_csv
+        assert not options.strip_url_path_trailing_slash
 
     @pytest.mark.parametrize('option_name', [
         'keep_blank_qs_values',
