@@ -1,11 +1,7 @@
 from datetime import date, datetime
+import json
 from uuid import UUID
 
-try:
-    import ujson as json
-except ImportError:
-    import json
-
 import pytest
 
 import falcon
@@ -65,12 +61,18 @@ def simulate_request_post_query_params(client, path, query_string, **kwargs):
     return client.simulate_request(path=path, body=query_string, **kwargs)
 
 
-@pytest.mark.parametrize('simulate_request', [
-    simulate_request_get_query_params,
-    simulate_request_post_query_params,
-])
-class TestQueryParams(object):
+@pytest.fixture(
+    scope='session',
+    params=[
+        simulate_request_get_query_params,
+        simulate_request_post_query_params,
+    ],
+)
+def simulate_request(request):
+    return request.param
 
+
+class TestQueryParams(object):
     def test_none(self, simulate_request, client, resource):
         query_string = ''
         client.app.add_route('/', resource)  # TODO: DRY up this setup logic
@@ -82,12 +84,30 @@ class TestQueryParams(object):
         assert req.get_param('limit', store) is None
         assert 'limit' not in store
         assert req.get_param_as_int('limit') is None
+        assert req.get_param_as_float('limit') is None
         assert req.get_param_as_bool('limit') is None
         assert req.get_param_as_list('limit') is None
 
+    def test_default(self, simulate_request, client, resource):
+        default = 'foobar'
+        query_string = ''
+        client.app.add_route('/', resource)  # TODO: DRY up this setup logic
+        simulate_request(client=client, path='/', query_string=query_string)
+
+        req = resource.captured_req
+        store = {}
+        assert req.get_param('marker', default=default) == 'foobar'
+        assert req.get_param('limit', store, default=default) == 'foobar'
+        assert 'limit' not in store
+        assert req.get_param_as_int('limit', default=default) == 'foobar'
+        assert req.get_param_as_float('limit', default=default) == 'foobar'
+        assert req.get_param_as_bool('limit', default=default) == 'foobar'
+        assert req.get_param_as_list('limit', default=default) == 'foobar'
+
     def test_blank(self, simulate_request, client, resource):
         query_string = 'marker='
         client.app.add_route('/', resource)
+        client.app.req_options.keep_blank_qs_values = False
         simulate_request(client=client, path='/', query_string=query_string)
 
         req = resource.captured_req
@@ -113,6 +133,7 @@ class TestQueryParams(object):
     def test_percent_encoded(self, simulate_request, client, resource):
         query_string = 'id=23,42&q=%e8%b1%86+%e7%93%a3'
         client.app.add_route('/', resource)
+        client.app.req_options.auto_parse_qs_csv = True
         simulate_request(client=client, path='/', query_string=query_string)
 
         req = resource.captured_req
@@ -140,6 +161,7 @@ class TestQueryParams(object):
     def test_option_auto_parse_qs_csv_simple_true(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         client.app.req_options.auto_parse_qs_csv = True
+        client.app.req_options.keep_blank_qs_values = False
 
         query_string = 'id=23,42,,&id=2'
         simulate_request(client=client, path='/', query_string=query_string)
@@ -153,6 +175,7 @@ class TestQueryParams(object):
     def test_option_auto_parse_qs_csv_complex_false(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         client.app.req_options.auto_parse_qs_csv = False
+        client.app.req_options.keep_blank_qs_values = False
 
         encoded_json = '%7B%22msg%22:%22Testing%201,2,3...%22,%22code%22:857%7D'
         decoded_json = '{"msg":"Testing 1,2,3...","code":857}'
@@ -181,6 +204,17 @@ class TestQueryParams(object):
 
         assert req.get_param('thing') == decoded_json
 
+    def test_default_auto_parse_csv_behaviour(self, simulate_request, client, resource):
+        client.app.add_route('/', resource=resource)
+        query_string = 'id=1,2,,&id=3'
+
+        simulate_request(client=client, path='/', query_string=query_string)
+
+        req = resource.captured_req
+
+        assert req.get_param('id') == '3'
+        assert req.get_param_as_list('id') == ['1,2,,', '3']
+
     def test_bad_percentage(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         query_string = 'x=%%20%+%&y=peregrine&z=%a%z%zz%1%20e'
@@ -194,6 +228,7 @@ class TestQueryParams(object):
 
     def test_allowed_names(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
+        client.app.req_options.keep_blank_qs_values = False
         query_string = ('p=0&p1=23&2p=foo&some-thing=that&blank=&'
                         'some_thing=x&-bogus=foo&more.things=blah&'
                         '_thing=42&_charset_=utf-8')
@@ -214,6 +249,7 @@ class TestQueryParams(object):
     @pytest.mark.parametrize('method_name', [
         'get_param',
         'get_param_as_int',
+        'get_param_as_float',
         'get_param_as_uuid',
         'get_param_as_bool',
         'get_param_as_list',
@@ -257,33 +293,33 @@ class TestQueryParams(object):
         assert req.get_param_as_int('limit', store=store) == 25
         assert store['limit'] == 25
 
-        assert req.get_param_as_int('limit', min=1, max=50) == 25
+        assert req.get_param_as_int('limit', min_value=1, max_value=50) == 25
 
         with pytest.raises(falcon.HTTPBadRequest):
-            req.get_param_as_int('limit', min=0, max=10)
+            req.get_param_as_int('limit', min_value=0, max_value=10)
 
         with pytest.raises(falcon.HTTPBadRequest):
-            req.get_param_as_int('limit', min=0, max=24)
+            req.get_param_as_int('limit', min_value=0, max_value=24)
 
         with pytest.raises(falcon.HTTPBadRequest):
-            req.get_param_as_int('limit', min=30, max=24)
+            req.get_param_as_int('limit', min_value=30, max_value=24)
 
         with pytest.raises(falcon.HTTPBadRequest):
-            req.get_param_as_int('limit', min=30, max=50)
+            req.get_param_as_int('limit', min_value=30, max_value=50)
 
-        assert req.get_param_as_int('limit', min=1) == 25
+        assert req.get_param_as_int('limit', min_value=1) == 25
 
-        assert req.get_param_as_int('limit', max=50) == 25
+        assert req.get_param_as_int('limit', max_value=50) == 25
 
-        assert req.get_param_as_int('limit', max=25) == 25
+        assert req.get_param_as_int('limit', max_value=25) == 25
 
-        assert req.get_param_as_int('limit', max=26) == 25
+        assert req.get_param_as_int('limit', max_value=26) == 25
 
-        assert req.get_param_as_int('limit', min=25) == 25
+        assert req.get_param_as_int('limit', min_value=25) == 25
 
-        assert req.get_param_as_int('limit', min=24) == 25
+        assert req.get_param_as_int('limit', min_value=24) == 25
 
-        assert req.get_param_as_int('limit', min=-24) == 25
+        assert req.get_param_as_int('limit', min_value=-24) == 25
 
     def test_int_neg(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
@@ -293,21 +329,96 @@ class TestQueryParams(object):
         req = resource.captured_req
         assert req.get_param_as_int('pos') == -7
 
-        assert req.get_param_as_int('pos', min=-10, max=10) == -7
+        assert req.get_param_as_int('pos', min_value=-10, max_value=10) == -7
 
-        assert req.get_param_as_int('pos', max=10) == -7
+        assert req.get_param_as_int('pos', max_value=10) == -7
 
         with pytest.raises(falcon.HTTPBadRequest):
-            req.get_param_as_int('pos', min=-6, max=0)
+            req.get_param_as_int('pos', min_value=-6, max_value=0)
 
         with pytest.raises(falcon.HTTPBadRequest):
-            req.get_param_as_int('pos', min=-6)
+            req.get_param_as_int('pos', min_value=-6)
 
         with pytest.raises(falcon.HTTPBadRequest):
-            req.get_param_as_int('pos', min=0, max=10)
+            req.get_param_as_int('pos', min_value=0, max_value=10)
 
         with pytest.raises(falcon.HTTPBadRequest):
-            req.get_param_as_int('pos', min=0, max=10)
+            req.get_param_as_int('pos', min_value=0, max_value=10)
+
+    def test_float(self, simulate_request, client, resource):
+        client.app.add_route('/', resource)
+        query_string = 'marker=deadbeef&limit=25.1'
+        simulate_request(client=client, path='/', query_string=query_string)
+
+        req = resource.captured_req
+
+        try:
+            req.get_param_as_float('marker')
+        except Exception as ex:
+            assert isinstance(ex, falcon.HTTPBadRequest)
+            assert isinstance(ex, falcon.HTTPInvalidParam)
+            assert ex.title == 'Invalid parameter'
+            expected_desc = ('The "marker" parameter is invalid. '
+                             'The value must be a float.')
+            assert ex.description == expected_desc
+
+        assert req.get_param_as_float('limit') == 25.1
+
+        store = {}
+        assert req.get_param_as_float('limit', store=store) == 25.1
+        assert store['limit'] == 25.1
+
+        assert req.get_param_as_float('limit', min_value=1, max_value=50) == 25.1
+
+        with pytest.raises(falcon.HTTPBadRequest):
+            req.get_param_as_float('limit', min_value=0, max_value=10)
+
+        with pytest.raises(falcon.HTTPBadRequest):
+            req.get_param_as_float('limit', min_value=0, max_value=24)
+
+        with pytest.raises(falcon.HTTPBadRequest):
+            req.get_param_as_float('limit', min_value=30, max_value=24)
+
+        with pytest.raises(falcon.HTTPBadRequest):
+            req.get_param_as_float('limit', min_value=30, max_value=50)
+
+        assert req.get_param_as_float('limit', min_value=1) == 25.1
+
+        assert req.get_param_as_float('limit', max_value=50) == 25.1
+
+        assert req.get_param_as_float('limit', max_value=25.1) == 25.1
+
+        assert req.get_param_as_float('limit', max_value=26) == 25.1
+
+        assert req.get_param_as_float('limit', min_value=25) == 25.1
+
+        assert req.get_param_as_float('limit', min_value=24) == 25.1
+
+        assert req.get_param_as_float('limit', min_value=-24) == 25.1
+
+    def test_float_neg(self, simulate_request, client, resource):
+        client.app.add_route('/', resource)
+        query_string = 'marker=deadbeef&pos=-7.1'
+        simulate_request(client=client, path='/', query_string=query_string)
+
+        req = resource.captured_req
+        assert req.get_param_as_float('pos') == -7.1
+
+        assert req.get_param_as_float('pos', min_value=-10, max_value=10) == -7.1
+
+        assert req.get_param_as_float('pos', max_value=10) == -7.1
+
+        with pytest.raises(falcon.HTTPBadRequest):
+            req.get_param_as_float('pos', min_value=-6, max_value=0)
+
+        with pytest.raises(falcon.HTTPBadRequest):
+            req.get_param_as_float('pos', min_value=-6)
+
+        with pytest.raises(falcon.HTTPBadRequest):
+            req.get_param_as_float('pos', min_value=0, max_value=10)
+
+        with pytest.raises(falcon.HTTPBadRequest):
+            req.get_param_as_float('pos', min_value=0, max_value=10)
 
     def test_uuid(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
@@ -338,6 +449,7 @@ class TestQueryParams(object):
 
     def test_boolean(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
+        client.app.req_options.keep_blank_qs_values = False
         query_string = ('echo=true&doit=false&bogus=bar&bogus2=foo&'
                         't1=True&f1=False&t2=yes&f2=no&blank&one=1&zero=0&'
                         'checkbox1=on&checkbox2=off')
@@ -377,22 +489,26 @@ class TestQueryParams(object):
 
     def test_boolean_blank(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
-        client.app.req_options.keep_blank_qs_values = True
         simulate_request(client=client, path='/', query_string='blank&blank2=')
 
         req = resource.captured_req
         assert req.get_param('blank') == ''
         assert req.get_param('blank2') == ''
-        with pytest.raises(falcon.HTTPInvalidParam):
-            req.get_param_as_bool('blank')
 
-        with pytest.raises(falcon.HTTPInvalidParam):
-            req.get_param_as_bool('blank2')
-        assert req.get_param_as_bool('blank', blank_as_true=True)
-        assert req.get_param_as_bool('blank3', blank_as_true=True) is None
+        for param_name in ('blank', 'blank2'):
+            assert req.get_param_as_bool(param_name) is True
+            assert req.get_param_as_bool(param_name, blank_as_true=True) is True
+            assert req.get_param_as_bool(param_name, blank_as_true=False) is False
+
+        assert req.get_param_as_bool('nichts') is None
+        assert req.get_param_as_bool('nichts', default=None) is None
+        assert req.get_param_as_bool('nichts', default=False) is False
+        assert req.get_param_as_bool('nichts', default=True) is True
 
     def test_list_type(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
+        client.app.req_options.auto_parse_qs_csv = True
+        client.app.req_options.keep_blank_qs_values = False
         query_string = ('colors=red,green,blue&limit=1'
                         '&list-ish1=f,,x&list-ish2=,0&list-ish3=a,,,b'
                         '&empty1=&empty2=,&empty3=,,'
@@ -443,6 +559,7 @@ class TestQueryParams(object):
                         '&empty4=&empty4&empty4='
                         '&empty5&empty5&empty5')
         client.app.req_options.keep_blank_qs_values = True
+        client.app.req_options.auto_parse_qs_csv = True
         simulate_request(client=client, path='/', query_string=query_string)
 
         req = resource.captured_req
@@ -482,6 +599,8 @@ class TestQueryParams(object):
 
     def test_list_transformer(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
+        client.app.req_options.auto_parse_qs_csv = True
+        client.app.req_options.keep_blank_qs_values = False
         query_string = 'coord=1.4,13,15.1&limit=100&things=4,,1'
         simulate_request(client=client, path='/', query_string=query_string)
 
@@ -551,6 +670,13 @@ class TestQueryParams(object):
         req = resource.captured_req
         assert req.get_param_as_int('ant') in (1, 2, 3)
 
+    def test_multiple_keys_as_float(self, simulate_request, client, resource):
+        client.app.add_route('/', resource)
+        query_string = 'ant=1.1&ant=2.2&ant=3.3'
+        simulate_request(client=client, path='/', query_string=query_string)
+        req = resource.captured_req
+        assert req.get_param_as_float('ant') in (1.1, 2.2, 3.3)
+
     def test_multiple_form_keys_as_list(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         query_string = 'ant=1&ant=2&bee=3&cat=6&cat=5&cat=4'
@@ -567,7 +693,7 @@ class TestQueryParams(object):
     def test_get_date_valid(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         date_value = '2015-04-20'
-        query_string = 'thedate={0}'.format(date_value)
+        query_string = 'thedate={}'.format(date_value)
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
         assert req.get_param_as_date('thedate') == date(2015, 4, 20)
@@ -582,7 +708,7 @@ class TestQueryParams(object):
     def test_get_date_valid_with_format(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         date_value = '20150420'
-        query_string = 'thedate={0}'.format(date_value)
+        query_string = 'thedate={}'.format(date_value)
         format_string = '%Y%m%d'
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
@@ -591,7 +717,7 @@ class TestQueryParams(object):
     def test_get_date_store(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         date_value = '2015-04-20'
-        query_string = 'thedate={0}'.format(date_value)
+        query_string = 'thedate={}'.format(date_value)
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
         store = {}
@@ -601,7 +727,7 @@ class TestQueryParams(object):
     def test_get_date_invalid(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         date_value = 'notarealvalue'
-        query_string = 'thedate={0}'.format(date_value)
+        query_string = 'thedate={}'.format(date_value)
         format_string = '%Y%m%d'
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
@@ -611,7 +737,7 @@ class TestQueryParams(object):
     def test_get_datetime_valid(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         date_value = '2015-04-20T10:10:10Z'
-        query_string = 'thedate={0}'.format(date_value)
+        query_string = 'thedate={}'.format(date_value)
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
         assert req.get_param_as_datetime('thedate') == datetime(2015, 4, 20, 10, 10, 10)
@@ -626,7 +752,7 @@ class TestQueryParams(object):
     def test_get_datetime_valid_with_format(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         date_value = '20150420 10:10:10'
-        query_string = 'thedate={0}'.format(date_value)
+        query_string = 'thedate={}'.format(date_value)
         format_string = '%Y%m%d %H:%M:%S'
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
@@ -636,7 +762,7 @@ class TestQueryParams(object):
     def test_get_datetime_store(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         datetime_value = '2015-04-20T10:10:10Z'
-        query_string = 'thedate={0}'.format(datetime_value)
+        query_string = 'thedate={}'.format(datetime_value)
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
         store = {}
@@ -647,7 +773,7 @@ class TestQueryParams(object):
     def test_get_datetime_invalid(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         date_value = 'notarealvalue'
-        query_string = 'thedate={0}'.format(date_value)
+        query_string = 'thedate={}'.format(date_value)
         format_string = '%Y%m%dT%H:%M:%S'
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
@@ -657,37 +783,50 @@ class TestQueryParams(object):
     def test_get_dict_valid(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         payload_dict = {'foo': 'bar'}
-        query_string = 'payload={0}'.format(json.dumps(payload_dict))
+        query_string = 'payload={}'.format(json.dumps(payload_dict))
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
-        assert req.get_param_as_dict('payload') == payload_dict
+        assert req.get_param_as_json('payload') == payload_dict
 
     def test_get_dict_missing_param(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         payload_dict = {'foo': 'bar'}
-        query_string = 'notthepayload={0}'.format(json.dumps(payload_dict))
+        query_string = 'notthepayload={}'.format(json.dumps(payload_dict))
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
-        assert req.get_param_as_dict('payload') is None
+        assert req.get_param_as_json('payload') is None
 
     def test_get_dict_store(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         payload_dict = {'foo': 'bar'}
-        query_string = 'payload={0}'.format(json.dumps(payload_dict))
+        query_string = 'payload={}'.format(json.dumps(payload_dict))
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
         store = {}
-        req.get_param_as_dict('payload', store=store)
+        req.get_param_as_json('payload', store=store)
         assert len(store) != 0
 
     def test_get_dict_invalid(self, simulate_request, client, resource):
         client.app.add_route('/', resource)
         payload_dict = 'foobar'
-        query_string = 'payload={0}'.format(payload_dict)
+        query_string = 'payload={}'.format(payload_dict)
         simulate_request(client=client, path='/', query_string=query_string)
         req = resource.captured_req
         with pytest.raises(HTTPInvalidParam):
-            req.get_param_as_dict('payload')
+            req.get_param_as_json('payload')
+
+    def test_has_param(self, simulate_request, client, resource):
+        client.app.add_route('/', resource)
+        query_string = 'ant=1'
+        simulate_request(client=client, path='/', query_string=query_string)
+
+        req = resource.captured_req
+        # There is a 'ant' key.
+        assert req.has_param('ant')
+        # There is not a 'bee' key..
+        assert not req.has_param('bee')
+        # There is not a None key
+        assert not req.has_param(None)
 
 
 class TestPostQueryParams(object):
