@@ -1,7 +1,4 @@
-try:
-    import ujson as json
-except ImportError:
-    import json
+import json
 
 import falcon
 from falcon import testing
@@ -13,10 +10,8 @@ class TypeResource(testing.SimpleTestResource):
     def on_post(self, req, resp, **kwargs):
         resp.status = falcon.HTTP_200
         # NOTE(masterkale): No size needs to be specified here because we're
-        # emulating a stream read in production. The request should be wrapped
-        # well enough to automatically specify a size when calling `read()`
-        # during either production or when running tests
-        resp.body = json.dumps({'data': req.stream.read().decode('utf-8')})
+        # emulating a stream read in production.
+        resp.body = json.dumps({'data': req.bounded_stream.read().decode('utf-8')})
 
 
 class TestWsgiRefInputWrapper(object):
