@@ -19,7 +19,7 @@ set -e
 VENV_NAME=tmp-falcon-build
 BUILD_DIR=./build
 DIST_DIR=./dist
-PY2_VERSION=2.7.14
+PY2_VERSION=2.7.16
 
 #----------------------------------------------------------------------
 # Helpers
@@ -33,8 +33,10 @@ _open_env() {
     pyenv virtualenv $PY_VERSION $VENV_NAME
     pyenv shell $VENV_NAME
 
-    pip install --upgrade pip
-    pip install --upgrade wheel twine
+    pip install -q --upgrade pip
+    pip install -q --upgrade wheel twine
+
+    echo
 }
 
 # Args: ()
@@ -75,19 +77,6 @@ rm -rf $DIST_DIR
 pyenv shell system
 pyenv uninstall -f $VENV_NAME
 
-
-#----------------------------------------------------------------------
-# README validation
-#----------------------------------------------------------------------
-
-_echo_task "Checking that README will render on PyPI"
-_open_env $PY2_VERSION
-
-pip install readme_renderer
-python setup.py check -r -s
-
-_close_env
-
 #----------------------------------------------------------------------
 # Source distribution
 #----------------------------------------------------------------------
@@ -100,7 +89,22 @@ python setup.py sdist -d $DIST_DIR
 _close_env
 
 #----------------------------------------------------------------------
-# Universal wheel - do not include Cython, note in README
+# manylinux wheels w/ Cython
+#----------------------------------------------------------------------
+
+_echo_task "Building manylinux wheels"
+
+DOCKER_IMAGE=quay.io/pypa/manylinux1_x86_64
+docker pull $DOCKER_IMAGE
+docker run --rm -v `pwd`:/io $DOCKER_IMAGE /io/tools/build-manylinux.sh
+
+DOCKER_IMAGE=quay.io/pypa/manylinux1_i686
+PRE_CMD=linux32
+docker pull $DOCKER_IMAGE
+docker run --rm -v `pwd`:/io $DOCKER_IMAGE $PRE_CMD /io/tools/build-manylinux.sh
+
+#----------------------------------------------------------------------
+# Universal wheel - Do not include Cython
 #----------------------------------------------------------------------
 
 _echo_task "Building universal wheel"
@@ -109,3 +113,14 @@ _open_env $PY2_VERSION
 python setup.py bdist_wheel -d $DIST_DIR
 
 _close_env
+
+#----------------------------------------------------------------------
+# README validation
+#----------------------------------------------------------------------
+
+_echo_task "Verifying PyPI compatibility"
+_open_env $PY2_VERSION
+
+twine check $DIST_DIR/*
+
+_close_env
